@@ -22,11 +22,17 @@ import Control.Monad (liftM2)
 import qualified Crypto.Hash.Algorithms as CHA
 import Crypto.Number.Serialize (i2osp)
 import qualified Crypto.PubKey.DSA as DSA
+import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
 import qualified Crypto.PubKey.RSA.PKCS15 as P15
 
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString.Lazy as BL
 import Data.Either (lefts, rights)
+#if MIN_VERSION_base(4,7,0)
+import Data.Either (isRight)
+#else
+import Control.Error.Util (isRight)
+#endif
 import Data.IxSet.Typed ((@=))
 import qualified Data.IxSet.Typed as IxSet
 import Data.List.NonEmpty (NonEmpty(..))
@@ -36,7 +42,7 @@ import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 import Data.Binary.Put (runPut)
 
 import Codec.Encryption.OpenPGP.Fingerprint (eightOctetKeyID, fingerprint)
-import Codec.Encryption.OpenPGP.Internal (PktStreamContext(..), issuer, emptyPSC, truncatingVerify)
+import Codec.Encryption.OpenPGP.Internal (PktStreamContext(..), issuer, emptyPSC)
 import Codec.Encryption.OpenPGP.Ontology (isRevokerP, isRevocationKeySSP, isSubkeyBindingSig, isSubkeyRevocation)
 
 import Codec.Encryption.OpenPGP.SerializeForSigs (putPartialSigforSigning, putSigTrailer, payloadForSig)
@@ -59,8 +65,8 @@ verifyTKWith :: (Pkt -> PktStreamContext -> Maybe UTCTime -> Either String Verif
 verifyTKWith vsf mt key = do
     revokers <- checkRevokers key
     revs <- checkKeyRevocations revokers key
-    let uids = filter (\(_, sps) -> sps /= []) . checkUidSigs $ key^.tkUIDs -- FIXME: check revocations here?
-    let uats = filter (\(_, sps) -> sps /= []) . checkUAtSigs $ key^.tkUAts -- FIXME: check revocations here?
+    let uids = filter (not . null . snd) . checkUidSigs $ key^.tkUIDs -- FIXME: check revocations here?
+    let uats = filter (not . null . snd) . checkUAtSigs $ key^.tkUAts -- FIXME: check revocations here?
     let subs = concatMap checkSub $ key^.tkSubs -- FIXME: check revocations here?
     return (TK (key^.tkKey) revs uids uats subs)
     where
@@ -92,9 +98,7 @@ verifyTKWith vsf mt key = do
         vSig :: SignaturePayload -> Either String Verification
         vSig sp = vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1) } mt
         vSubSig :: Pkt -> SignaturePayload -> Bool
-        vSubSig sk sp = case vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1), lastSubkey = sk} mt of
-                                Left _ -> False
-                                Right _ -> True
+        vSubSig sk sp = isRight (vsf (SignaturePkt sp) emptyPSC { lastPrimaryKey = PublicKeyPkt (key ^. tkKey._1), lastSubkey = sk} mt)
         verifyRevoker :: SignaturePayload -> Either String [(PubKeyAlgorithm, TwentyOctetFingerprint)]
         verifyRevoker sp = do
             _ <- vSig sp
@@ -128,14 +132,17 @@ verifyAgainstKeys ks sig mt payload = do
         verify' (SignaturePkt s) (pub@(PKPayload V4 _ _ _ pkey)) SHA224 pl = verify'' (pkaAndMPIs s) CHA.SHA224 pub pkey pl
         verify' (SignaturePkt s) (pub@(PKPayload V4 _ _ _ pkey)) DeprecatedMD5 pl = verify'' (pkaAndMPIs s) CHA.MD5 pub pkey pl
         verify' _ _ _ _ = error "This should never happen (verify')."
-        verify'' (DSA,mpis) hd pub (DSAPubKey (DSA_PublicKey pkey)) bs = verify''' (dsaVerify mpis hd pkey bs) pub
-        verify'' (RSA,mpis) hd pub (RSAPubKey (RSA_PublicKey pkey)) bs = verify''' (rsaVerify mpis hd pkey bs) pub
+        verify'' (DSA,mpis) hd pub (DSAPubKey (DSA_PublicKey pkey)) bs = dsaVerify pub mpis hd pkey bs
+        verify'' (ECDSA,mpis) hd pub (ECDSAPubKey (ECDSA_PublicKey pkey)) bs = ecdsaVerify pub mpis hd pkey bs
+        verify'' (RSA,mpis) hd pub (RSAPubKey (RSA_PublicKey pkey)) bs = rsaVerify pub mpis hd pkey bs
         verify'' _ _ _ _ _ = Left "unimplemented key type"
-        verify''' f pub = if f then Right pub else Left "verification failed"
-        dsaVerify (r:|[s]) hd pkey = truncatingVerify hd pkey (dsaMPIsToSig r s)
-        dsaVerify _ _ _ = const False -- FIXME: this should be some sort of Either chain?
-        rsaVerify mpis hd pkey bs = P15.verify (Just hd) pkey bs (rsaMPItoSig mpis)
+        dsaVerify pub (r:|[s]) hd pkey bs = if DSA.verify hd pkey (dsaMPIsToSig r s) bs then Right pub else Left ("DSA verification failed: " ++ show (hd,pkey,r,s,bs))
+        dsaVerify _ _ _ _ _ = Left "cannot verify DSA signature of wrong shape"
+        ecdsaVerify pub (r:|[s]) hd pkey bs = if ECDSA.verify hd pkey (ecdsaMPIsToSig r s) bs then Right pub else Left ("ECDSA verification failed: " ++ show (hd,pkey,r,s,bs))
+        ecdsaVerify _ _ _ _ _ = Left "cannot verify ECDSA signature of wrong shape"
+        rsaVerify pub mpis hd pkey bs = if P15.verify (Just hd) pkey bs (rsaMPItoSig mpis) then Right pub else Left ("DSA verification failed: " ++ show (hd,pkey,mpis,bs))
         dsaMPIsToSig r s = DSA.Signature (unMPI r) (unMPI s)
+        ecdsaMPIsToSig r s = ECDSA.Signature (unMPI r) (unMPI s)
         rsaMPItoSig (s:|[]) = i2osp (unMPI s)
         hashalgo :: Pkt -> HashAlgorithm
         hashalgo (SignaturePkt (SigV4 _ _ ha _ _ _ _)) = ha
