@@ -27,7 +27,6 @@ import Control.Newtype (Newtype(..))
 import Data.Aeson ((.=), object)
 import qualified Data.Aeson as A
 import qualified Data.Aeson.TH as ATH
-import Data.Byteable (Byteable)
 import Data.ByteArray (ByteArrayAccess)
 import Data.ByteString.Lazy (ByteString)
 import qualified Data.ByteString as B
@@ -869,7 +868,7 @@ instance Pretty KeyVersion where
 $(ATH.deriveJSON ATH.defaultOptions ''KeyVersion)
 
 newtype IV = IV {unIV :: B.ByteString}
-    deriving (Byteable, ByteArrayAccess, Data, Eq, Generic, Hashable, Monoid, Show, Typeable)
+    deriving (ByteArrayAccess, Data, Eq, Generic, Hashable, Monoid, Show, Typeable)
 
 instance Newtype IV B.ByteString where
     pack = IV
@@ -915,7 +914,7 @@ instance Pretty DataType where
 $(ATH.deriveJSON ATH.defaultOptions ''DataType)
 
 newtype Salt = Salt {unSalt :: B.ByteString}
-    deriving (Byteable, Data, Eq, Generic, Hashable, Show, Typeable)
+    deriving (Data, Eq, Generic, Hashable, Show, Typeable)
 
 instance Newtype Salt B.ByteString where
     pack = Salt
