@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 
 -- suite.hs: hOpenPGP test suite
--- Copyright © 2012-2016  Clint Adams
+-- Copyright © 2012-2018  Clint Adams
 -- This software is released under the terms of the Expat license.
 -- (See the LICENSE file).
 
@@ -24,7 +24,8 @@ import Codec.Encryption.OpenPGP.Serialize (parsePkts)
 import Codec.Encryption.OpenPGP.Signatures (verifyTKWith, verifySigWith, verifyAgainstKeys)
 import Codec.Encryption.OpenPGP.Types
 import Control.Error.Util (isRight)
-import Control.Monad.Trans.Resource (ResourceT, runResourceT)
+import Control.Monad.Trans.Resource (ResourceT)
+import qualified Crypto.PubKey.ECC.ECDSA as ECDSA
 import qualified Crypto.PubKey.RSA as RSA
 import Data.Conduit.Serialization.Binary (conduitGet)
 import Data.Conduit.OpenPGP.Compression (conduitCompress, conduitDecompress)
@@ -76,12 +77,12 @@ testCompression fpr = do
                           else
                               assertEqual ("for " ++ fpr) firstpass secondpass
 
-counter :: (Monad m) => DC.Sink a m Int
+counter :: (Monad m) => DC.ConduitT a DC.Void m Int
 counter = CL.fold (const . (1+)) 0
 
-testConduitOutputLength :: FilePath -> DC.Conduit B.ByteString (ResourceT IO) b -> Int -> Assertion
+testConduitOutputLength :: FilePath -> DC.ConduitT B.ByteString b (ResourceT IO) () -> Int -> Assertion
 testConduitOutputLength fpr c target = do
-    len <- runResourceT $ CB.sourceFile ("tests/data/" ++ fpr) DC.$= c DC.$$ counter
+    len <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ fpr) DC..| c DC..| counter
     assertEqual ("expected length " ++ show target) target len
 
 testPKAandSizeAndKeyIDandFingerprint :: FilePath -> String -> Assertion
@@ -99,26 +100,26 @@ testPKAandSizeAndKeyIDandFingerprint fpr kf = do
 
 testKeyringLookup :: FilePath -> String -> Bool -> Assertion
 testKeyringLookup fpr eok expected = do
-    kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ fpr) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ sinkKeyringMap
+    kr <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ fpr) DC..| conduitGet get DC..| conduitToTKs DC..| sinkKeyringMap
     let key = getOne (kr @= (read eok :: EightOctetKeyId))
     assertEqual (eok ++ " in " ++ fpr) expected (isJust key)
 
 testVerifyMessage :: FilePath -> FilePath -> [TwentyOctetFingerprint] -> Assertion
 testVerifyMessage keyring message issuers = do
-    kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyring) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ sinkKeyringMap
-    verification <- runResourceT $ CB.sourceFile ("tests/data/" ++ message) DC.$= conduitGet get DC.$= conduitDecompress DC.$= conduitVerify kr Nothing DC.$$ CL.consume
+    kr <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyring) DC..| conduitGet get DC..| conduitToTKs DC..| sinkKeyringMap
+    verification <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ message) DC..| conduitGet get DC..| conduitDecompress DC..| conduitVerify kr Nothing DC..| CL.consume
     let verification' = map (fmap (fingerprint . _verificationSigner)) verification
     assertEqual (keyring ++ " for " ++ message) (map Right issuers) verification'
 
 testKeysSelfVerification :: Bool -> FilePath -> Assertion
 testKeysSelfVerification expectsuccess keyfile = do
-    ks <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ CL.consume
+    ks <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyfile) DC..| conduitGet get DC..| conduitToTKs DC..| CL.consume
     let verifieds = mapM (verifyTKWith (verifySigWith (verifyAgainstKeys ks)) Nothing) ks
     assertEqual (keyfile ++ " self-verification") expectsuccess (isRight verifieds)
 
 testKeysExpiration :: Bool -> FilePath -> Assertion
 testKeysExpiration expectsuccess keyfile = do
-    ks <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$= conduitToTKs DC.$$ CL.consume
+    ks <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyfile) DC..| conduitGet get DC..| conduitToTKs DC..| CL.consume
     let Right verifieds = mapM (verifyTKWith (verifySigWith (verifyAgainstKeys ks)) Nothing) ks
         tvalid = all (isTKTimeValid (posixSecondsToUTCTime (realToFrac (1400000000 :: Integer)))) verifieds
     assertEqual (keyfile ++ " key expiration") expectsuccess tvalid
@@ -128,7 +129,7 @@ testSymmetricEncryption :: FilePath -> FilePath -> BL.ByteString -> Assertion
 testSymmetricEncryption encfile passfile cleartext = do
   passphrase <- BL.readFile $ "tests/data/" ++ passfile
   -- get parse tree
-  pt <- runResourceT $ CB.sourceFile ("tests/data/" ++ encfile) DC.$= conduitGet get DC.$$ CL.consume
+  pt <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ encfile) DC..| conduitGet get DC..| CL.consume
   -- assert parse tree has exactly two packets: skesk, encdata
   assertEqual "wrong number of packets" 2 (length pt)
   let skesk = fromPkt.head $ pt
@@ -138,7 +139,7 @@ testSymmetricEncryption encfile passfile cleartext = do
   -- and the type system chokes before we hit them:
   assertEqual "first packet should be SKESK" SKESKType (packetType skesk)
   assertEqual "second packet should be encrypted data" SymEncIntegrityProtectedDataType (packetType d)
-  decrypted <- runResourceT $ CL.sourceList pt DC.$= conduitDecrypt (fakeCallback passphrase) DC.$$ CL.consume
+  decrypted <- DC.runConduitRes $ CL.sourceList pt DC..| conduitDecrypt (fakeCallback passphrase) DC..| CL.consume
   let payload = _literalDataPayload . fromPkt . head $ decrypted
   assertEqual ("cleartext for " ++ encfile) cleartext payload
       where
@@ -148,7 +149,7 @@ testSymmetricEncryption encfile passfile cleartext = do
 testSecretKeyDecryption :: FilePath -> FilePath -> Assertion
 testSecretKeyDecryption keyfile passfile = do
   passphrase <- BL.readFile $ "tests/data/" ++ passfile
-  kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
+  kr <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyfile) DC..| conduitGet get DC..| CL.consume
   let SecretKey pkp ska = fromPkt . head $ kr
       SUUnencrypted skey _ = decryptPrivateKey (pkp, ska) passphrase
   doPkeyAndSkeyMatch (_pubkey pkp) skey
@@ -157,8 +158,8 @@ testSecretKeyDecryption keyfile passfile = do
 testSecretKeyEncryption :: FilePath -> FilePath -> Assertion
 testSecretKeyEncryption keyfile passfile = do
   passphrase <- BL.readFile $ "tests/data/" ++ passfile
-  kr <- runResourceT $ CB.sourceFile ("tests/data/" ++ keyfile) DC.$= conduitGet get DC.$$ CL.consume
-  gkr <- runResourceT $ CB.sourceFile ("tests/data/" ++ "aes256-sha512.seckey") DC.$= conduitGet get DC.$$ CL.consume
+  kr <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ keyfile) DC..| conduitGet get DC..| CL.consume
+  gkr <- DC.runConduitRes $ CB.sourceFile ("tests/data/" ++ "aes256-sha512.seckey") DC..| conduitGet get DC..| CL.consume
   let SecretKey pkp ska = fromPkt . head $ kr
       newska = encryptPrivateKey "\226~\197\a\202#\"G" (IV "\187\219\253I\236\204\t5D\196\NAK>;\202\185\t") ska passphrase
       newtruck = toPkt (SecretKey pkp newska):tail kr
@@ -167,7 +168,7 @@ testSecretKeyEncryption keyfile passfile = do
 testParsePktsUtil :: FilePath -> Assertion
 testParsePktsUtil fn = do
     let fpath = "tests/data/" ++ fn
-    cp <- runResourceT $ CB.sourceFile fpath DC.$= conduitGet get DC.$$ CL.consume
+    cp <- DC.runConduitRes $ CB.sourceFile fpath DC..| conduitGet get DC..| CL.consume
     pp <- parsePkts `fmap` BL.readFile fpath
     assertEqual "parsePkts utility function gives same results as conduit pipeline" cp pp
 
@@ -175,7 +176,7 @@ testParseTKsUtil :: FilePath -> Assertion
 testParseTKsUtil fn = do
     let fpath = "tests/data/" ++ fn
     lbs <- BL.readFile fpath
-    cp <- runResourceT $ CB.sourceLbs lbs DC.$= conduitGet get DC.$= conduitToTKs DC.$$ CL.consume
+    cp <- DC.runConduitRes $ CB.sourceLbs lbs DC..| conduitGet get DC..| conduitToTKs DC..| CL.consume
     let pt = parseTKs True . parsePkts $ lbs
     assertEqual "parsePkts utility function gives same results as conduit pipeline" cp pt
 
@@ -276,10 +277,13 @@ unitTests = testGroup "Unit Tests" [
    , testCase "sigs-with-regexes" (testSerialization "sigs-with-regexes")
    , testCase "gnu-dummy-s2k-101-secret-key.gpg" (testSerialization "gnu-dummy-s2k-101-secret-key.gpg")
    , testCase "anibal-ed25519.gpg" (testSerialization "anibal-ed25519.gpg")
+   , testCase "nist_p-256_key.gpg" (testSerialization "nist_p-256_key.gpg")
+   , testCase "nist_p-256_secretkey.gpg" (testSerialization "nist_p-256_secretkey.gpg")
    ],
   testGroup "PKA/Size/KeyID/fingerprint group" [
      testCase "v3 key" (testPKAandSizeAndKeyIDandFingerprint "v3.key" "R1024:C7261095/CBD9 F412 6807 E405 CC2D  2712 1DF5 E86E")
    , testCase "v4 key" (testPKAandSizeAndKeyIDandFingerprint "000001-006.public_key" "R1248:D4D54EA16F87040E/421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E")
+   , testCase "ECDSA key" (testPKAandSizeAndKeyIDandFingerprint "nist_p-256_key.gpg" "E256:F7708BADD6063224/174C CF12 C571 6D0E 527F  B50E F770 8BAD D606 3224")
    ],
   testGroup "Keyring group" [
      testCase "pubring 7732CF988A63EA86" (testKeyringLookup "pubring.gpg" "7732CF988A63EA86" True)
@@ -308,15 +312,18 @@ unitTests = testGroup "Unit Tests" [
    , testCase "6F87040E" (testVerifyMessage "pubring.gpg" "6F87040E.pubkey" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"])
    , testCase "6F87040E-cr" (testVerifyMessage "pubring.gpg" "6F87040E-cr.pubkey" [fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC", fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E", fp "CB79 3345 9F59 C70D F1C3  FBEE DEDC 3ECF 689A F56D", fp "AF95 E4D7 BAC5 21EE 9740  BED7 5E9F 1523 4132 62DC"])
    , testCase "simple RSA secret key" (testVerifyMessage "pubring.gpg" "simple.seckey" [fp "421F 28FE AAD2 22F8 56C8  FFD5 D4D5 4EA1 6F87 040E"])
+   , testCase "simple ECDSA public key" (testVerifyMessage "ecdsa-key-without-ecdh.pubkey" "ecdsa-key-without-ecdh.pubkey" [fp "174C CF12 C571 6D0E 527F  B50E F770 8BAD D606 3224"])
    ],
   testGroup "Key verification group" [
       testCase "6F87040E pubkey" (testKeysSelfVerification True "6F87040E.pubkey")
    ,  testCase "revoked pubkey" (testKeysSelfVerification False "revoked.pubkey")
    ,  testCase "expired pubkey" (testKeysSelfVerification True "expired.pubkey")
+   ,  testCase "nist_p-256 pubkey" (testKeysSelfVerification True "nist_p-256_key.gpg")
    ],
   testGroup "Key expiration group" [
       testCase "6F87040E pubkey" (testKeysExpiration True "6F87040E.pubkey")
    ,  testCase "expired pubkey" (testKeysExpiration False "expired.pubkey")
+   ,  testCase "nist_p-256 pubkey" (testKeysExpiration True "nist_p-256_key.gpg")
    ],
   testGroup "Compression group" [
      testCase "compressedsig.gpg" (testCompression "compressedsig.gpg")
@@ -324,11 +331,11 @@ unitTests = testGroup "Unit Tests" [
    , testCase "compressedsig-bzip2.gpg" (testCompression "compressedsig-bzip2.gpg")
    ],
   testGroup "Conduit length group" [
-     testCase "conduitCompress (ZIP)" (testConduitOutputLength "pubring.gpg" (cgp DC.=$= conduitCompress ZIP) 1)
-   , testCase "conduitCompress (Zlib)" (testConduitOutputLength "pubring.gpg" (cgp DC.=$= conduitCompress ZLIB) 1)
-   , testCase "conduitCompress (BZip2)" (testConduitOutputLength "pubring.gpg" (cgp DC.=$= conduitCompress BZip2) 1)
-   , testCase "conduitToTKs" (testConduitOutputLength "pubring.gpg" (cgp DC.=$= conduitToTKs) 4)
-   , testCase "conduitToTKsDropping" (testConduitOutputLength "pubring.gpg" (cgp DC.=$= conduitToTKsDropping) 4)
+     testCase "conduitCompress (ZIP)" (testConduitOutputLength "pubring.gpg" (cgp DC..| conduitCompress ZIP) 1)
+   , testCase "conduitCompress (Zlib)" (testConduitOutputLength "pubring.gpg" (cgp DC..| conduitCompress ZLIB) 1)
+   , testCase "conduitCompress (BZip2)" (testConduitOutputLength "pubring.gpg" (cgp DC..| conduitCompress BZip2) 1)
+   , testCase "conduitToTKs" (testConduitOutputLength "pubring.gpg" (cgp DC..| conduitToTKs) 4)
+   , testCase "conduitToTKsDropping" (testConduitOutputLength "pubring.gpg" (cgp DC..| conduitToTKsDropping) 4)
    ],
   testGroup "Encrypted data" [
      testCase "Symmetric Encryption simple S2K SHA1 3DES, no MDC" (testSymmetricEncryption "encryption-sym-3des-s2k0.gpg" "symmetric-password.txt" "test\n")
@@ -360,6 +367,7 @@ unitTests = testGroup "Unit Tests" [
      testCase "SUSSHA1 CAST5 IteratedSalted SHA1 RSA" (testSecretKeyDecryption "simple.seckey" "pki-password.txt")
    , testCase "SUS16bit CAST5 IteratedSalted SHA1 RSA" (testSecretKeyDecryption "16bitcksum.seckey" "pki-password.txt")
    , testCase "SUSSHA1 AES256 IteratedSalted SHA512 RSA" (testSecretKeyDecryption "aes256-sha512.seckey" "pki-password.txt")
+   , testCase "SUSSHA1 AES128 IteratedSalted SHA256 ECDSA" (testSecretKeyDecryption "nist_p-256_secretkey.gpg" "pki-password.txt")
    ],
   testGroup "Encrypting secret keys" [
      testCase "SUSSHA1 AES256 IteratedSalted SHA512 RSA" (testSecretKeyEncryption "unencrypted.seckey" "pki-password.txt")
@@ -383,7 +391,7 @@ qcProps = testGroup "(checked by QuickCheck)"
     \uid -> Right (uid :: UserId) == runGet get (runPut (put uid))
   ]
 
-cgp :: DC.Conduit B.ByteString (ResourceT IO) Pkt
+cgp :: DC.ConduitT B.ByteString Pkt (ResourceT IO) ()
 cgp = conduitGet (get :: Get Pkt)
 
 fp :: Text -> TwentyOctetFingerprint
@@ -391,6 +399,7 @@ fp = either error id . parseFingerprint
 
 doPkeyAndSkeyMatch :: PKey -> SKey -> Assertion
 doPkeyAndSkeyMatch (RSAPubKey (RSA_PublicKey rpub)) (RSAPrivateKey (RSA_PrivateKey rpriv)) = assertEqual "RSA private key matches RSA public key" rpub (RSA.private_pub rpriv)
+doPkeyAndSkeyMatch (ECDSAPubKey (ECDSA_PublicKey ecpub)) (ECDSAPrivateKey (ECDSA_PrivateKey ecpriv)) = assertEqual "ECDSA private key curve matches ECDSA public key curve" (ECDSA.public_curve ecpub) (ECDSA.private_curve ecpriv)
 doPkeyAndSkeyMatch _ _ = assertFailure "matching unimplemented"
 
 main :: IO ()
